#!/usr/bin/guile-3.0 \
-e main -s
!#

;; Copyright (C) 2008-2018, 2021 Tommi Höynälänmaa
;; Distributed under GNU General Public License version 3,
;; see file doc/GPL-3.


(import (rnrs exceptions))
(import (rnrs conditions))
(import (rnrs base))
(import (theme-d common theme-d-config))
(import (theme-d testing test-info))
(import (th-scheme-utilities hrecord))
(import (th-scheme-utilities parse-command-line))


(define (get-test-name i)
  (string-append "test" (number->string i)))


(define (get-source-filename i s-unit-type)
  (let ((str-ext (if (eq? s-unit-type 'script) ".tcs" ".tcp")))
    (string-append "test" (number->string i) str-ext)))


(define (get-interm-filename i s-intermediate-language)
  (let ((s-ext
	 (case s-intermediate-language
	   ((tree-il) ".tree-il")
	   ((guile) ".scm")
	   ((guile0) ".scm")
	   (else (raise 'invalid-intermediate-language)))))
    (string-append "test" (number->string i) s-ext)))


(define (get-target-filename i)
  (string-append "test" (number->string i) ".go"))


(define (get-test-path filename)
  (string-append "../theme-d-code/tests/" filename))


(define (link-program index output-file str-link-command
		      s-intermediate-language keep-intermediate?
		      str-extra-link-options)
  (assert (integer? index))
  (assert (output-port? output-file))
  (assert (and (string? str-link-command)
	       (not (string-null? str-link-command))))
  (assert (and (symbol? s-intermediate-language)
	       (memq s-intermediate-language '(tree-il guile guile0))))
  (assert (boolean? keep-intermediate?))
  (assert (string? str-extra-link-options))
  (let* ((s-unit-type (if (member index scripts) 'script 'proper-program))
	 (source-filename (get-source-filename index s-unit-type))
	 (interm-filename (get-interm-filename index
					       s-intermediate-language))
	 (str-interm-type
	  (case s-intermediate-language
	    ((tree-il)
	     "-i tree-il")
	    ((guile)
	     "-i guile")
	    ((guile0)
	     (string-append "-i guile --no-optimization"))
	    (else (raise 'invalid-intermediate-language))))
	 (str-keep-option
	  (if keep-intermediate?
	      (string-append
	       "--keep-intermediate -n "
	       (get-test-path interm-filename))
	      ""))
	 (target-filename (get-target-filename index)))
    (display (string-append "Linking program "))
    (display source-filename)
    (newline)
    (flush-all-ports)
    (let ((source-path (get-test-path source-filename))
	  (target-path (get-test-path target-filename))
	  (str-extra-modules
	   (cond
	    ((member index custom1-programs)
	     "-x \"(theme-d testing custom1)\"")
	    ((member index custom2-programs)
	     "-x \"(theme-d testing custom2)\"")
	    (else ""))))
      (let ((str-command
	     (string-append
	      str-link-command
	      " -m ../theme-d-code: "
	      str-interm-type
	      " "
	      str-keep-option
	      " "
	      str-extra-modules
	      " "
	      str-extra-link-options
	      " -o "
	      target-path
	      " " source-path " 2>&1")))
	(display str-command)
	(newline)
	(let ((result (system str-command)))
	  (if (= result 0)
	      (begin
		(display "Linking of " output-file)
		(display source-filename output-file)
		(display " succeeded." output-file)
		(newline output-file))
	      (begin
		(display "Linking of " output-file)
		(display source-filename output-file)
		(display " failed." output-file)
		(newline output-file))))))))


(define (link-programs output-file str-link-command
		       s-intermediate-language keep-intermediate?
		       str-extra-link-options)
  (do ((i 1 (+ i 1))) ((> i nr-of-tests))
    (if (not (member i modules))
  	(link-program i output-file str-link-command
		      s-intermediate-language keep-intermediate?
		      str-extra-link-options))))


(define (main args)
  (init-theme-d-config)
  (let* ((str1 (getenv "EXTRA_LINK_OPTIONS"))
	 (str-extra-link-options (if str1 str1 ""))
	 (str-link-command0 "")
	 (s-intermediate-language 'tree-il)
	 (keep-intermediate? #f)
	 (argd1 (make-hrecord <argument-descriptor> #\b #t
			      (lambda (str-option-arg)
				(set! str-link-command0 str-option-arg))))
	 (argd2 (make-hrecord <argument-descriptor> #\i #f
			      (lambda (str-option-arg)
				(cond
				 ((equal? str-option-arg "tree-il")
				  (set! s-intermediate-language 'tree-il))
				 ((equal? str-option-arg "guile")
				  (set! s-intermediate-language 'guile))
				 ((equal? str-option-arg "guile0")
				  (set! s-intermediate-language 'guile0))
				 (else
				  (display "Invalid intermediate language.\n")
				  (exit 1))))))
	 (argd3 (make-hrecord <argument-descriptor> #\k #f
			      (lambda ()
				(set! keep-intermediate? #t))))
	 (l-arg-descs (list argd1 argd2 argd3))
	 (handle-proper-args
	  (lambda (l-proper-args)
	    (if (> (length l-proper-args) 0)
		(begin
		  (display "Extra arguments in command line. Exiting.\n")
		  (exit 1)))
	    #f))
	 (l-args-without-cmd (cdr args)))
    (parse-command-line l-args-without-cmd l-arg-descs handle-proper-args)
    (let ((str-link-command
	   (if (not (string-null? str-link-command0))
	       (string-append "run-theme-d-program " str-link-command0)
	       (get-theme-d-config-var 'linker-path))))
      (call-with-output-file "link-test-programs.log"
	(lambda (output-file)
	  (link-programs output-file str-link-command s-intermediate-language
			 keep-intermediate? str-extra-link-options))))))
